{% extends "index.html" %}
{% block indexuniq %} <title> Aggrescan3D: How to </title> {% endblock %}


{% block body %}
<div class="container">
    <div class="row">
        <div class="col-md-12 hyphenate">
            {% include "tutorial_text.html" %}
        </div>
    </div>
</div>
{% endblock %}
{%block below_js%}
{%endblock %}
