{% extends "index.html" %}
{% block indexuniq %} <title> Aggrescan3D method </title> {% endblock %}


{% block body %}
<div class="container">
    <div class="row">
        <div class="col-md-12 hyphenate">
            {% include "learn_more_text.html" %}
        </div>
    </div>
</div>
{% endblock %}
{%block below_js%}
<script type="text/javascript" src="http://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML"></script>
{%endblock %}
