{%extends "index.html" %}
{% block indexuniq %} <title> Aggrescan3D: How to cite </title> {% endblock %}

{%block body %}
<div class="container">
    <div class="row">
        <div class="col-md-12">
            <h1>Citation</h1>
                                        Rafael Zambrano, Michal Jamroz, Agata Szczasiuk, Jordi Pujols, Sebastian Kmiecik, and Salvador Ventura
                                            <strong>AGGRESCAN3D (A3D): server for prediction of aggregation in protein structures</strong>, 
                                            Nucleic Acids Research,
                                            <i>submitted</i> 
        </div>
    </div>
</div>
{%endblock%}
{%block below_js%}
{%endblock %}
