{% extends "index.html" %}
{% block indexuniq %} <title> Aggrescan3D: Contact</title> {% endblock %}


{% block body %}
<div class="container">
    <div class="row">
        <div class="col-md-12">
            <address>
            <strong>Laboratory of Theory of Biopolymers</strong><br>
            Faculty of Chemistry,
            University of Warsaw<br>
            Pasteura 1, 02-093 Warsaw, Poland<br>
            tel.: (+48) 22 8220211<br>
            fax: (+48) 22 8220221, (+48) 22 8225996<br>
            <a href="http://biocomp.chem.uw.edu.pl" target="_blank">Laboratory homepage</a><br>

            <i class="fa fa-envelope-square"></i> sekmi<i class="fa fa-at"></i>chem.uw.edu.pl

            </address>
            <address>
                <strong> 
                Institut de Biotecnologia i Biomedicina and Departament de Bioquímica i Biologia Molecular
                </strong>
                <br>Universitat Autònoma de Barcelona<br> Bellaterra 08193, Spain<br> 
                tel.:  93 586 89 56; tel. laboratori:   93 586 89 54 <br>fax: 93 581 20 11  <br>
            <a href=http://ibb.uab.es/ibb/ibb_membres/view_grup.php?CodiGrup=36 target=_blank>Laboratory homepage</a><br>

            <i class="fa fa-envelope-square"></i> salvador.ventura<i class="fa fa-at"></i>uab.es
        </address>

        </div>
    </div>
    <div class="row">
        <div class="hyphenate col-md-12">
            <hr>
            <h2>Citation</h2>
                                        Rafael Zambrano, Michal Jamroz, Agata Szczasiuk, Jordi Pujols, Sebastian Kmiecik, and Salvador Ventura
                                            <strong>AGGRESCAN3D (A3D): server for prediction of aggregation properties of protein structures</strong>, 
                                            Nucleic Acids Research, 2015, 43(W1) W306-W313 doi: 10.1093/nar/gkv359
        </div>
</div>
    <div class="row">
        <div class="col-md-12">
            <hr>
            <h2>Funding</h2>
<p>
This work was supported by Ministerio de Economia y Competividad, Spain [BFU2013-44763 to S.V.]; by SUDOE, INTERREG IV B, FEDER [SOE4/P1/E831to S.V.]; by ICREA [ICREA Academia 2009 to S.V.] and Foundation for Polish Science TEAM project [TEAM/2011-7/6] co-financed by the EU European Regional Development Fund operated within the Innovative Economy Operational Program.  
</p>
        </div>
</div>

{% endblock %}
{%block below_js%}
{%endblock %}
